@@ -10,7 +10,7 @@ import re
 import tempfile
 import time
 
-from mercurial import util, error, scmutil, phases
+from mercurial import util, error, phases
 from mercurial import obsolete  # delete if obsolete becomes enabled by default
 
 from tortoisehg.util import hglib, i18n, shlib, wconfig
@@ -132,13 +132,13 @@ _topicmap = {
 class CommitWidget(QWidget, qtlib.TaskWidget):
     'A widget that encompasses a StatusWidget and commit extras'
     commitButtonEnable = pyqtSignal(bool)
-    linkActivated = pyqtSignal(QString)
-    showMessage = pyqtSignal(unicode)
-    grepRequested = pyqtSignal(unicode, dict)
+    linkActivated = pyqtSignal(str)
+    showMessage = pyqtSignal(str)
+    grepRequested = pyqtSignal(str, dict)
     runCustomCommandRequested = pyqtSignal(str, list)
     commitComplete = pyqtSignal()
 
-    progress = pyqtSignal(QString, object, QString, QString, object)
+    progress = pyqtSignal(str, object, str, str, object)
 
     def __init__(self, repoagent, pats, opts, parent=None, rev=None):
         QWidget.__init__(self, parent)
@@ -464,7 +464,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             elif curraction._name == 'commit':
                 refreshwctx = refresh and oldpctx is not None
                 self.stwidget.setPatchContext(None)
-                allowcs = len(self.repo.parents()) == 1
+                allowcs = len(self.repo[None].parents()) == 1
         if curraction._name in ('qref', 'amend'):
             if self.lastAction not in ('qref', 'amend'):
                 self.lastCommitMsgs['commit'] = (self.msgte.text(),
@@ -511,7 +511,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         branch = hglib.fromunicode(self.branchop)
         if branch in repo.branchmap():
             # response: 0=Yes, 1=No, 2=Cancel
-            if branch in [p.branch() for p in repo.parents()]:
+            if branch in [p.branch() for p in repo[None].parents()]:
                 resp = 0
             else:
                 rev = repo[branch].rev()
@@ -565,7 +565,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         wholecmdlines.extend(cmdlines)
         self._runCommand(curraction._name, wholecmdlines)
 
-    @pyqtSlot(QString, QString)
+    @pyqtSlot(str, str)
     def fileDisplayed(self, wfile, contents):
         'Status widget is displaying a new file'
         if not (wfile and contents):
@@ -633,7 +633,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
     def details(self):
         mode = 'commit'
-        if len(self.repo.parents()) > 1:
+        if len(self.repo[None].parents()) > 1:
             mode = 'merge'
         dlg = DetailsDialog(self._repoagent, self.opts, self.userhist, self,
                             mode=mode)
@@ -740,6 +740,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                     text = engmsg and msgset['id'] or msgset['str']
                     self.setMessage(unicode(text) %
                                     hglib.tounicode(self.repo[None].branch()))
+            self.msgte.setFocus()
             self.refresh()
 
     def canUndo(self):
@@ -815,12 +816,12 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.userhist = map(unicode, s.value(gpref+'userhist').toStringList())
         self.userhist = [u for u in self.userhist if u]
         try:
-            curmsg = self.repo.opener('cur-message.txt').read()
+            curmsg = self.repo.vfs('cur-message.txt').read()
             self.setMessage(hglib.tounicode(curmsg))
         except EnvironmentError:
             pass
         try:
-            curmsg = self.repo.opener('last-message.txt').read()
+            curmsg = self.repo.vfs('last-message.txt').read()
             if curmsg:
                 self.addMessageToHistory(hglib.tounicode(curmsg))
         except EnvironmentError:
@@ -838,7 +839,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             s.setValue(gpref+'history-'+repoid, self.msghistory)
             s.setValue(gpref+'userhist', self.userhist)
             msg = self.getMessage(True)
-            self.repo.opener('cur-message.txt', 'w').write(msg)
+            self.repo.vfs('cur-message.txt', 'w').write(msg)
         except (EnvironmentError, IOError):
             pass
 
@@ -864,8 +865,8 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         except UnicodeEncodeError:
             res = qtlib.CustomPrompt(
                     _('Message Translation Failure'),
-                    _('Unable to translate message to local encoding\n'
-                      'Consider setting HGENCODING environment variable\n'
+                    _('Unable to translate message to local encoding.\n'
+                      'Consider setting HGENCODING environment variable.\n\n'
                       'Replace untranslatable characters with "?"?\n'), self,
                      (_('&Replace'), _('Cancel')), 0, 1, []).run()
             if res == 0:
@@ -918,12 +919,12 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             if commandlines is None:
                 return
         partials = []
-        if len(repo.parents()) > 1:
+        if len(repo[None].parents()) > 1:
             merge = True
             self.files = []
         else:
             merge = False
-            files = self.stwidget.getChecked('MAR?!S')
+            files = self.stwidget.getChecked('MAR?!IS')
             # make list of files with partial change selections
             for fname, c in self.stwidget.partials.iteritems():
                 if c.excludecount > 0 and c.excludecount < len(c.hunks):
@@ -937,8 +938,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.stwidget.tv.setFocus()
             return
 
+        # username will be prompted as necessary by hg if ui.askusername
         user = self.opts.get('user')
-        if not amend:
+        if not amend and not repo.ui.configbool('ui', 'askusername'):
             # TODO: no need to specify --user if it was read from ui
             user = qtlib.getCurrentUsername(self, self.repo, self.opts)
             if not user:
@@ -946,6 +948,16 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.addUsernameToHistory(user)
 
         checkedUnknowns = self.stwidget.getChecked('?I')
+        if checkedUnknowns and 'largefiles' in repo.extensions():
+            result = lfprompt.promptForLfiles(self, repo.ui, repo,
+                                              checkedUnknowns)
+            if not result:
+                return
+            checkedUnknowns, lfiles = result
+            if lfiles:
+                cmd = ['add', '--large', '--']
+                cmd.extend(map(hglib.escapepath, lfiles))
+                commandlines.append(cmd)
         if checkedUnknowns:
             confirm = self.repo.ui.configbool('tortoisehg',
                                               'confirmaddfiles', True)
@@ -958,21 +970,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             else:
                 res = 0
             if res == 0:
-                haslf = 'largefiles' in repo.extensions()
-                if haslf:
-                    result = lfprompt.promptForLfiles(self, repo.ui, repo,
-                                                      checkedUnknowns)
-                    if not result:
-                        return
-                    checkedUnknowns, lfiles = result
-                    if lfiles:
-                        cmd = ['add', '--large', '--']
-                        cmd.extend(map(hglib.escapepath, lfiles))
-                        commandlines.append(cmd)
-                if checkedUnknowns:
-                    cmd = ['add', '--']
-                    cmd.extend(map(hglib.escapepath, checkedUnknowns))
-                    commandlines.append(cmd)
+                cmd = ['add', '--']
+                cmd.extend(map(hglib.escapepath, checkedUnknowns))
+                commandlines.append(cmd)
             else:
                 return
         checkedMissing = self.stwidget.getChecked('!')
@@ -993,25 +993,13 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
                 commandlines.append(cmd)
             else:
                 return
-        try:
-            date = self.opts.get('date')
-            if date:
-                util.parsedate(date)
-                dcmd = ['--date', date]
-            else:
-                dcmd = []
-        except error.Abort, e:
-            if e.hint:
-                err = _('%s (hint: %s)') % (hglib.tounicode(str(e)),
-                                            hglib.tounicode(e.hint))
-            else:
-                err = hglib.tounicode(str(e))
-            self.showMessage.emit(err)
-            dcmd = []
         cmdline = ['commit', '--verbose', '--message='+msg]
         if user:
             cmdline.extend(['--user', user])
-        cmdline += dcmd + brcmd
+        date = self.opts.get('date')
+        if date:
+            cmdline += ['--date', date]
+        cmdline += brcmd
 
         if partials:
             # write patch for partial change selections to temp file
@@ -1042,7 +1030,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
 
         cmdline.append('--')
         cmdline.extend(map(hglib.escapepath, self.files))
-        if len(repo.parents()) == 1:
+        if len(repo[None].parents()) == 1:
             for fname in self.opts.get('autoinc', '').split(','):
                 fname = fname.strip()
                 if fname:
@@ -1256,7 +1244,7 @@ class DetailsDialog(QDialog):
         self.saveToPath([fn])
 
     def saveGlobal(self):
-        self.saveToPath(scmutil.userrcpath())
+        self.saveToPath(hglib.userrcpath())
 
     def saveToPath(self, path):
         fn, cfg = hgrcutil.loadIniFile(path, self)
