@@ -13,7 +13,7 @@ from mercurial import ui, hg, error, commands, match, util, subrepo
 from tortoisehg.hgqt import htmlui, visdiff, qtlib, htmldelegate, thgrepo, cmdui, settings
 from tortoisehg.hgqt import filedialogs, fileview
 from tortoisehg.util import paths, hglib, thread2
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -23,8 +23,8 @@ from PyQt4.QtGui import *
 
 class SearchWidget(QWidget, qtlib.TaskWidget):
     '''Working copy and repository search widget'''
-    showMessage = pyqtSignal(QString)
-    progress = pyqtSignal(QString, object, QString, QString, object)
+    showMessage = pyqtSignal(str)
+    progress = pyqtSignal(str, object, str, str, object)
     revisionSelected = pyqtSignal(int)
 
     def __init__(self, repoagent, upats, parent=None, **opts):
@@ -100,12 +100,7 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         grid.setColumnStretch(1, 0)
         frame = QFrame()
         frame.setFrameStyle(QFrame.StyledPanel)
-        def revisiontoggled(checked):
-            revle.setEnabled(checked)
-            if checked:
-                revle.selectAll()
-                QTimer.singleShot(0, lambda:revle.setFocus())
-        revision.toggled.connect(revisiontoggled)
+        revision.toggled.connect(self._onRevisionToggled)
         history.toggled.connect(singlematch.setDisabled)
         revle.setEnabled(False)
         revle.returnPressed.connect(self.runSearch)
@@ -113,30 +108,13 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         incle.returnPressed.connect(self.runSearch)
         bt.clicked.connect(self.runSearch)
 
-        def updateRecurse(checked):
-            try:
-                wctx = self.repo[None]
-                if '.hgsubstate' in wctx:
-                    recurse.setEnabled(checked)
-                else:
-                    recurse.setEnabled(False)
-                    recurse.setChecked(False)
-            except Exception:
-                recurse.setEnabled(False)
-                recurse.setChecked(False)
-        working.toggled.connect(updateRecurse)
         recurse.setChecked(True)
         working.setChecked(True)
+        working.toggled.connect(self._updateRecurse)
 
-        def updatefollow():
-            slowpath = bool(incle.text() or excle.text())
-            follow.setEnabled(history.isChecked() and not slowpath)
-            if slowpath:
-                follow.setChecked(False)
-        history.toggled.connect(updatefollow)
-        incle.textChanged.connect(updatefollow)
-        excle.textChanged.connect(updatefollow)
-        updatefollow()
+        history.toggled.connect(self._updateFollow)
+        incle.textChanged.connect(self._updateFollow)
+        excle.textChanged.connect(self._updateFollow)
 
         mainvbox.addLayout(hbox)
         frame.setLayout(grid)
@@ -165,7 +143,7 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
             incle.setText(','.join(upats[1:]))
         chk.setChecked(opts.get('ignorecase', False))
 
-        repoid = str(repo[0])
+        repoid = hglib.shortrepoid(repo)
         s = QSettings()
         sh = list(s.value('grep/search-'+repoid).toStringList())
         ph = list(s.value('grep/paths-'+repoid).toStringList())
@@ -176,6 +154,9 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         mainvbox.setContentsMargins(0, 0, 0, 0)
         self.setLayout(mainvbox)
 
+        self._updateRecurse()
+        self._updateFollow()
+
     @property
     def repo(self):
         return self._repoagent.rawRepo()
@@ -226,6 +207,34 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         if isinstance(rev, int):
             self.revle.setText(str(rev))
 
+    @pyqtSlot(bool)
+    def _onRevisionToggled(self, checked):
+        self.revle.setEnabled(checked)
+        if checked:
+            self.revle.selectAll()
+            self.revle.setFocus()
+
+    @pyqtSlot()
+    def _updateRecurse(self):
+        checked = self.wctxradio.isChecked()
+        try:
+            wctx = self.repo[None]
+            if '.hgsubstate' in wctx:
+                self.recurse.setEnabled(checked)
+            else:
+                self.recurse.setEnabled(False)
+                self.recurse.setChecked(False)
+        except util.Abort:
+            self.recurse.setEnabled(False)
+            self.recurse.setChecked(False)
+
+    @pyqtSlot()
+    def _updateFollow(self):
+        slowpath = bool(self.incle.text() or self.excle.text())
+        self.follow.setEnabled(self.aradio.isChecked() and not slowpath)
+        if slowpath:
+            self.follow.setChecked(False)
+
     def setSearch(self, upattern, **opts):
         self.regexple.setText(upattern)
         if opts.get('all'):
@@ -253,7 +262,7 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         return True
 
     def saveSettings(self, s):
-        repoid = str(self.repo[0])
+        repoid = hglib.shortrepoid(self.repo)
         s.setValue('grep/search-'+repoid, self.searchhistory)
         s.setValue('grep/paths-'+repoid, self.pathshistory)
 
@@ -282,9 +291,9 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         self.tv.icase = icase
         self.regexple.selectAll()
         inc = hglib.fromunicode(self.incle.text())
-        if inc: inc = inc.split(', ')
+        if inc: inc = map(str.strip, inc.split(','))
         exc = hglib.fromunicode(self.excle.text())
-        if exc: exc = exc.split(', ')
+        if exc: exc = map(str.strip, exc.split(','))
         rev = hglib.fromunicode(self.revle.text()).strip()
 
         self.addHistory(pattern, inc or [], exc or [])
@@ -326,10 +335,6 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
                      lambda wrapper: model.appendRow(*wrapper.data))
         self.thread.start()
 
-    def reload(self):
-        # TODO
-        pass
-
     def searchfinished(self):
         self.cancelbutton.setEnabled(False)
         self.searchbutton.setEnabled(True)
@@ -355,8 +360,8 @@ class DataWrapper(object):
 class HistorySearchThread(QThread):
     '''Background thread for searching repository history'''
     matchedRow = pyqtSignal(DataWrapper)
-    showMessage = pyqtSignal(unicode)
-    progress = pyqtSignal(QString, object, QString, QString, object)
+    showMessage = pyqtSignal(str)
+    progress = pyqtSignal(str, object, str, str, object)
 
     def __init__(self, repo, pattern, icase, inc, exc, follow):
         super(HistorySearchThread, self).__init__()
@@ -418,7 +423,7 @@ class HistorySearchThread(QThread):
                     'rev':[], 'line_number':True, 'print0':True,
                     'ignore_case':self.icase, 'include':self.inc,
                     'exclude':self.exc}
-            u = incrui()
+            u = incrui(self.repo.ui)
             commands.grep(u, self.repo, self.pattern, **opts)
         except Exception, e:
             self.showMessage.emit(str(e))
@@ -431,8 +436,8 @@ class HistorySearchThread(QThread):
 class CtxSearchThread(QThread):
     '''Background thread for searching a changectx'''
     matchedRow = pyqtSignal(object)
-    showMessage = pyqtSignal(unicode)
-    progress = pyqtSignal(QString, object, QString, QString, object)
+    showMessage = pyqtSignal(str)
+    progress = pyqtSignal(str, object, str, str, object)
 
     def __init__(self, repo, regexp, ctx, inc, exc, once, recurse):
         super(CtxSearchThread, self).__init__()
@@ -453,7 +458,7 @@ class CtxSearchThread(QThread):
         def badfn(f, msg):
             e = hglib.tounicode("%s: %s" % (matchfn.rel(f), msg))
             self.showMessage.emit(e)
-        self.hu = htmlui.htmlui()
+        self.hu = htmlui.htmlui(self.repo.ui)
         try:
             # generate match function relative to repo root
             matchfn = match.match(self.repo.root, '', [], self.inc, self.exc)
@@ -536,6 +541,7 @@ class MatchTree(QTableView):
         self.selectedRows = ()
 
         self.delegate = htmldelegate.HTMLDelegate(self)
+        self.setDragDropMode(QTableView.DragOnly)
         self.setItemDelegateForColumn(COL_TEXT, self.delegate)
         self.setSelectionMode(QTableView.ExtendedSelection)
         self.setSelectionBehavior(QAbstractItemView.SelectRows)
@@ -566,51 +572,13 @@ class MatchTree(QTableView):
         self.activated.connect(self.onRowActivated)
         self.customContextMenuRequested.connect(self.menuRequest)
 
-        self.setModel(MatchModel(self))
+        self.setModel(MatchModel(repoagent, self))
         self.selectionModel().selectionChanged.connect(self.onSelectionChanged)
 
     @property
     def repo(self):
         return self._repoagent.rawRepo()
 
-    def dragObject(self):
-        snapshots = {}
-        for index in self.selectionModel().selectedRows():
-            path, line, rev, user, text = self.model().getRow(index)
-            if rev not in snapshots:
-                snapshots[rev] = [path]
-            else:
-                snapshots[rev].append(path)
-        urls = []
-        for rev, paths in snapshots.iteritems():
-            if rev is not None:
-                base, _ = visdiff.snapshot(self.repo, paths, self.repo[rev])
-            else:
-                base = self.repo.root
-            for p in paths:
-                urls.append(QUrl.fromLocalFile(os.path.join(base, path)))
-        if urls:
-            d = QDrag(self)
-            m = QMimeData()
-            m.setUrls(urls)
-            d.setMimeData(m)
-            d.start(Qt.CopyAction)
-
-    def mousePressEvent(self, event):
-        self.pressPos = event.pos()
-        self.pressTime = QTime.currentTime()
-        return QTableView.mousePressEvent(self, event)
-
-    def mouseMoveEvent(self, event):
-        d = event.pos() - self.pressPos
-        if d.manhattanLength() < QApplication.startDragDistance():
-            return QTableView.mouseMoveEvent(self, event)
-        elapsed = self.pressTime.msecsTo(QTime.currentTime())
-        if elapsed < QApplication.startDragTime():
-            return QTableView.mouseMoveEvent(self, event)
-        self.dragObject()
-        return QTableView.mouseMoveEvent(self, event)
-
     def menuRequest(self, point):
         if not self.selectionModel().selectedRows():
             return
@@ -653,10 +621,9 @@ class MatchTree(QTableView):
                 abs = repo.wjoin(path)
                 root = paths.find_root(abs)
                 if root and abs.startswith(root):
+                    uroot = hglib.tounicode(root)
+                    srepoagent = self._repoagent.subRepoAgent(uroot)
                     path = abs[len(root)+1:]
-                    # TODO: do not instantiate repo here
-                    srepo = thgrepo.repository(None, root)
-                    srepoagent = srepo._pyqtobj
                     self._openAnnotateDialog(srepoagent, rev, path, line)
                 else:
                     continue
@@ -726,26 +693,27 @@ class MatchTree(QTableView):
 
 
 class MatchModel(QAbstractTableModel):
-    def __init__(self, parent):
+    def __init__(self, repoagent, parent):
         QAbstractTableModel.__init__(self, parent)
+        self._repoagent = repoagent
         self.rows = []
         self.headers = (_('File'), _('Line'), _('Rev'), _('User'),
                         _('Match Text'))
 
-    def rowCount(self, parent):
+    def rowCount(self, parent=QModelIndex()):
         return len(self.rows)
 
-    def columnCount(self, parent):
+    def columnCount(self, parent=QModelIndex()):
         return len(self.headers)
 
-    def data(self, index, role):
+    def data(self, index, role=Qt.DisplayRole):
         if not index.isValid():
             return QVariant()
         if role == Qt.DisplayRole:
             return QVariant(self.rows[index.row()][index.column()])
         return QVariant()
 
-    def headerData(self, col, orientation, role):
+    def headerData(self, col, orientation, role=Qt.DisplayRole):
         if role != Qt.DisplayRole or orientation != Qt.Horizontal:
             return QVariant()
         else:
@@ -755,6 +723,34 @@ class MatchModel(QAbstractTableModel):
         flags = Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsDragEnabled
         return flags
 
+    def mimeTypes(self):
+        return ['text/uri-list']
+
+    def mimeData(self, indexes):
+        snapshots = {}
+        for index in indexes:
+            if index.column() != 0:
+                continue
+            path, line, rev, user, text = self.rows[index.row()]
+            if rev not in snapshots:
+                snapshots[rev] = [path]
+            else:
+                snapshots[rev].append(path)
+        urls = []
+        for rev, paths in snapshots.iteritems():
+            if rev is not None:
+                repo = self._repoagent.rawRepo()
+                lpaths = map(hglib.fromunicode, paths)
+                lbase, _ = visdiff.snapshot(repo, lpaths, repo[rev])
+                base = hglib.tounicode(lbase)
+            else:
+                base = self._repoagent.rootPath()
+            for p in paths:
+                urls.append(QUrl.fromLocalFile(os.path.join(base, p)))
+        m = QMimeData()
+        m.setUrls(urls)
+        return m
+
     def sort(self, col, order):
         self.layoutAboutToBeChanged.emit()
         self.rows.sort(key=lambda x: x[col],
