@@ -12,6 +12,12 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import QApplication, QFont
 from PyQt4.QtNetwork import QLocalServer, QLocalSocket
 
+if os.name == 'nt' and getattr(sys, 'frozen', False):
+    # load QtSvg4.dll and QtXml4.dll by .pyd, so that imageformats/qsvg4.dll
+    # can find them without relying on unreliable PATH variable
+    from PyQt4 import QtSvg, QtXml
+    QtSvg.__name__, QtXml.__name__  # no demandimport, silence pyflakes
+
 if PYQT_VERSION < 0x40600 or QT_VERSION < 0x40600:
     sys.stderr.write('TortoiseHg requires at least Qt 4.6 and PyQt 4.6\n')
     sys.stderr.write('You have Qt %s and PyQt %s\n' %
@@ -25,6 +31,10 @@ from tortoisehg.util import hglib, i18n
 from tortoisehg.util import version as thgversion
 from tortoisehg.hgqt import bugreport, qtlib, thgrepo, workbench
 
+if getattr(sys, 'frozen', False) and os.name == 'nt':
+    # load icons and translations
+    import icons_rc, translations_rc
+
 try:
     from thginithook import thginithook
 except ImportError:
@@ -117,13 +127,12 @@ class ExceptionCatcher(QObject):
     def excepthandler(self):
         'Display exception info; run in main (GUI) thread'
         try:
-            try:
-                self._showexceptiondialog()
-            except:
-                # make sure to quit mainloop first, so that it never leave
-                # zombie process.
-                self._mainapp.exit(1)
-                self._printexception()
+            self._showexceptiondialog()
+        except:
+            # make sure to quit mainloop first, so that it never leave
+            # zombie process.
+            self._mainapp.exit(1)
+            self._printexception()
         finally:
             self.errors = []
 
@@ -159,9 +168,9 @@ class ExceptionCatcher(QObject):
         # may show modal confirmation dialog in closeEvent().
         QTimer.singleShot(0, self._mainapp.closeAllWindows)
 
-    if os.name == 'posix' and util.safehasattr(signal, 'set_wakeup_fd'):
+    if os.name == 'posix':
         # Wake up Python interpreter via pipe so that SIGINT can be handled
-        # immediately.  (http://qt-project.org/doc/qt-4.8/unix-signals.html)
+        # immediately.  (https://doc.qt.io/qt-4.8/unix-signals.html)
 
         def _initWakeup(self):
             import fcntl
@@ -199,7 +208,7 @@ class ExceptionCatcher(QObject):
 
         def _initWakeup(self):
             self._wakeuptimer = 0
-            if self._ui.interactive():
+            if self._ui._isatty(self._ui.fin):
                 self._wakeuptimer = self.startTimer(200)
 
         def _releaseWakeup(self):
@@ -325,7 +334,7 @@ def _fixapplicationfont():
 
 def _gettranslationpath():
     """Return path to Qt's translation file (.qm)"""
-    if getattr(sys, 'frozen', False):
+    if getattr(sys, 'frozen', False) and os.name == 'nt':
         return ':/translations'
     else:
         return QLibraryInfo.location(QLibraryInfo.TranslationsPath)
@@ -335,7 +344,7 @@ class QtRunner(QObject):
 
     NOTE: This object will be instantiated before QApplication, it means
     there's a limitation on Qt's event handling. See
-    http://doc.qt.nokia.com/4.6/threads-qobject.html#per-thread-event-loop
+    https://doc.qt.io/qt-4.8/threads-qobject.html#per-thread-event-loop
     """
 
     def __init__(self):
@@ -382,7 +391,7 @@ class QtRunner(QObject):
         _fixapplicationfont()
         qtlib.configstyles(ui)
         qtlib.initfontcache(ui)
-        self._mainapp.setWindowIcon(qtlib.geticon('thg-logo'))
+        self._mainapp.setWindowIcon(qtlib.geticon('thg'))
 
         self._repomanager = thgrepo.RepoManager(ui, self)
         self._reporeleaser = releaser = QSignalMapper(self)
@@ -436,7 +445,7 @@ class QtRunner(QObject):
 
     def _fixlibrarypaths(self):
         # make sure to use the bundled Qt plugins to avoid ABI incompatibility
-        # http://qt-project.org/doc/qt-4.8/deployment-windows.html#qt-plugins
+        # https://doc.qt.io/qt-4.8/deployment-windows.html#qt-plugins
         if os.name == 'nt' and getattr(sys, 'frozen', False):
             self._mainapp.setLibraryPaths([self._mainapp.applicationDirPath()])
 
@@ -492,7 +501,7 @@ class QtRunner(QObject):
         self._workbench = workbench.Workbench(self._ui, self._repomanager)
         return self._workbench
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def openRepoInWorkbench(self, uroot):
         """Show the specified repository in Workbench; reuses the existing
         Workbench process"""
