@@ -62,19 +62,21 @@ class RebaseDialog(QDialog):
         sep = qtlib.LabeledSeparator(_('Options'))
         self.layout().addWidget(sep)
 
-        self.keepchk = QCheckBox(_('Keep original changesets'))
+        self.keepchk = QCheckBox(_('Keep original changesets (--keep)'))
         self.keepchk.setChecked(opts.get('keep', False))
         self.layout().addWidget(self.keepchk)
 
-        self.keepbrancheschk = QCheckBox(_('Keep original branch names'))
+        self.keepbrancheschk = QCheckBox(_('Keep original branch names '
+                                           '(--keepbranches)'))
         self.keepbrancheschk.setChecked(opts.get('keepbranches', False))
         self.layout().addWidget(self.keepbrancheschk)
 
-        self.collapsechk = QCheckBox(_('Collapse the rebased changesets'))
+        self.collapsechk = QCheckBox(_('Collapse the rebased changesets '
+                                       '(--collapse)'))
         self.collapsechk.setChecked(opts.get('collapse', False))
         self.layout().addWidget(self.collapsechk)
 
-        self.basechk = QCheckBox(_('Rebase entire source branch'))
+        self.basechk = QCheckBox(_('Rebase entire source branch (-b/--base)'))
         self.layout().addWidget(self.basechk)
 
         self.autoresolvechk = QCheckBox(_('Automatically resolve merge '
@@ -147,9 +149,10 @@ class RebaseDialog(QDialog):
     def _onCheckFinished(self, clean):
         if not clean:
             self.rebasebtn.setEnabled(False)
-            txt = _('Before rebase, you must <a href="commit">'
-                    '<b>commit</b></a> or <a href="discard">'
-                    '<b>discard</b></a> changes.')
+            txt = _('Before rebase, you must '
+                    '<a href="commit"><b>commit</b></a>, '
+                    '<a href="shelve"><b>shelve</b></a> to patch, '
+                    'or <a href="discard"><b>discard</b></a> changes.')
         else:
             self.rebasebtn.setEnabled(True)
             txt = _('You may continue the rebase')
@@ -157,7 +160,7 @@ class RebaseDialog(QDialog):
 
     def rebase(self):
         self.rebasebtn.setEnabled(False)
-        self.cancelbtn.setShown(False)
+        self.cancelbtn.setVisible(False)
         self.keepchk.setEnabled(False)
         self.keepbrancheschk.setEnabled(False)
         self.basechk.setEnabled(False)
@@ -166,7 +169,7 @@ class RebaseDialog(QDialog):
 
         itool = self.autoresolvechk.isChecked() and 'merge' or 'fail'
         opts = {'config': 'ui.merge=internal:%s' % itool}
-        if os.path.exists(self.repo.join('rebasestate')):
+        if os.path.exists(self.repo.vfs.join('rebasestate')):
             opts['continue'] = True
         else:
             opts.update({
@@ -246,7 +249,7 @@ class RebaseDialog(QDialog):
             txt = _('You may continue the rebase')
         self._stbar.showMessage(txt)
 
-        if os.path.exists(self.repo.join('rebasestate')):
+        if os.path.exists(self.repo.vfs.join('rebasestate')):
             self.swaplabel.setVisible(False)
             self.abortbtn.setEnabled(True)
             self.rebasebtn.setText('Continue')
@@ -264,7 +267,7 @@ class RebaseDialog(QDialog):
             self._wctxcleaner.runCleaner(cmd)
 
     def reject(self):
-        if os.path.exists(self.repo.join('rebasestate')):
+        if os.path.exists(self.repo.vfs.join('rebasestate')):
             main = _('Exiting with an unfinished rebase is not recommended.')
             text = _('Consider aborting the rebase first.')
             labels = ((QMessageBox.Yes, _('&Exit')),
