@@ -10,7 +10,7 @@ import os
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
-from mercurial import error, util, revset as hgrevset
+from mercurial import error, util
 from mercurial import repoview
 
 from tortoisehg.util import hglib
@@ -22,8 +22,9 @@ _permanent_queries = ('head()', 'merge()',
                       'file(".hgsubstate") or file(".hgsub")')
 
 def _firstword(query):
+    lquery = hglib.fromunicode(query)
     try:
-        for token, value, _pos in hgrevset.tokenize(hglib.fromunicode(query)):
+        for token, value, _pos in hglib.tokenizerevspec(lquery):
             if token == 'symbol' or token == 'string':
                 return value  # localstr
     except error.ParseError:
@@ -68,6 +69,24 @@ def _querytype(repo, query):
         pass
     return 'keyword'
 
+
+class SelectAllLineEdit(QLineEdit):
+    def __init__(self, parent=None):
+        super(SelectAllLineEdit, self).__init__(parent)
+        self.just_got_focus = False
+
+    def focusInEvent(self, ev):
+        if ev.reason() == Qt.MouseFocusReason:
+            self.just_got_focus = True
+        super(SelectAllLineEdit, self).focusInEvent(ev)
+
+    def mouseReleaseEvent(self, ev):
+        if self.just_got_focus:
+            self.just_got_focus = False
+            self.selectAll()
+        super(SelectAllLineEdit, self).mouseReleaseEvent(ev)
+
+
 class RepoFilterBar(QToolBar):
     """Toolbar for RepoWidget to filter changesets"""
 
@@ -89,10 +108,10 @@ class RepoFilterBar(QToolBar):
         self._repoagent = repoagent
         self._permanent_queries = list(_permanent_queries)
         repo = repoagent.rawRepo()
-        username = repo.ui.config('ui', ['username', 'user'])
+        username = hglib.configuredusername(repo.ui)
         if username:
             self._permanent_queries.insert(0,
-                hgrevset.formatspec('author(%s)', os.path.expandvars(username)))
+                hglib.formatrevspec('author(%s)', os.path.expandvars(username)))
         self.filterEnabled = True
 
         #Check if the font contains the glyph needed by the branch combo
@@ -105,8 +124,13 @@ class RepoFilterBar(QToolBar):
 
         self.revsetcombo = combo = QComboBox()
         combo.setEditable(True)
+        # don't calculate size hint from history contents, just use as much
+        # space as possible. this way, the branch combo can be enlarged up
+        # to its preferred width.
+        combo.setSizeAdjustPolicy(
+            QComboBox.AdjustToMinimumContentsLengthWithIcon)
         combo.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
-        combo.setMinimumContentsLength(10)
+        combo.setMinimumContentsLength(20)
         qtlib.allowCaseChangingInput(combo)
         le = combo.lineEdit()
         le.returnPressed.connect(self.runQuery)
@@ -238,8 +262,7 @@ class RepoFilterBar(QToolBar):
     def _prepareQuery(self):
         query = unicode(self.revsetcombo.currentText()).strip()
         if _querytype(self._repo, query) == 'keyword':
-            s = hglib.fromunicode(query)
-            return hglib.tounicode(hgrevset.formatspec('keyword(%s)', s))
+            return hglib.formatrevspec('keyword(%s)', query)
         else:
             return query
 
@@ -350,9 +373,17 @@ class RepoFilterBar(QToolBar):
         self._branchLabel.setMenu(self._branchMenu)
 
         self._branchCombo = QComboBox()
+        self._branchCombo.setEditable(True)
+        self._branchCombo.setInsertPolicy(QComboBox.NoInsert)
+        self._branchCombo.setSizeAdjustPolicy(QComboBox.AdjustToContents)
+        self._branchCombo.setLineEdit(SelectAllLineEdit())
+        self._branchCombo.lineEdit().editingFinished.connect(
+            self._lineBranchChanged)
         self._branchCombo.setMinimumContentsLength(10)
         self._branchCombo.setMaxVisibleItems(30)
         self._branchCombo.currentIndexChanged.connect(self._emitBranchChanged)
+        completer = QCompleter(self._branchCombo.model(), self._branchCombo)
+        self._branchCombo.setCompleter(completer)
 
         self.addWidget(self._branchLabel)
         self.addWidget(qtlib.Spacer(2, 2))
@@ -406,9 +437,10 @@ class RepoFilterBar(QToolBar):
             self._branchCombo.setItemData(i, branch, Qt.ToolTipRole)
             self._branchCombo.setItemData(i, branch)
         self._branchCombo.setEnabled(self.filterEnabled and bool(branches))
+        self.setBranch(curbranch)
         self._branchCombo.blockSignals(False)
 
-        if not self.setBranch(curbranch):
+        if self.branch() != curbranch:
             self._emitBranchChanged()  # falls back to "show all"
 
     @pyqtSlot(str)
@@ -417,8 +449,6 @@ class RepoFilterBar(QToolBar):
         index = self._branchCombo.findData(branch)
         if index >= 0:
             self._branchCombo.setCurrentIndex(index)
-            return True
-        return False
 
     def branch(self):
         """Return the current branch name [unicode]"""
@@ -440,6 +470,12 @@ class RepoFilterBar(QToolBar):
         self.branchChanged.emit(self.branch(),
                                 self.branchAncestorsIncluded())
 
+    @pyqtSlot()
+    def _lineBranchChanged(self):
+        written_branch = self._branchCombo.currentText()
+        if not written_branch:
+            self._branchCombo.setCurrentIndex(0)
+
     @pyqtSlot()
     def refresh(self):
         self._updateBranchFilter()
