@@ -7,7 +7,7 @@
 
 import os
 
-from mercurial import commands, hg, ui, util
+from mercurial import commands, hg, util
 
 from tortoisehg.util import hglib, paths
 from tortoisehg.util.i18n import _
@@ -25,16 +25,14 @@ def settingsfilename():
 
 class RepoTreeView(QTreeView):
     showMessage = pyqtSignal(str)
-    menuRequested = pyqtSignal(object, object)
-    openRepo = pyqtSignal(str, bool)
+    openRequested = pyqtSignal(QModelIndex)
+    removeRequested = pyqtSignal(QModelIndex)
     dropAccepted = pyqtSignal()
-    updateSettingsFile = pyqtSignal()
 
     def __init__(self, parent):
         QTreeView.__init__(self, parent, allColumnsShowFocus=True)
         if qtlib.IS_RETINA:
             self.setIconSize(qtlib.treeviewRetinaIconSize())
-        self.selitem = None
         self.msg = ''
 
         self.setHeaderHidden(True)
@@ -42,7 +40,8 @@ class RepoTreeView(QTreeView):
         self.setMouseTracking(True)
 
         # enable drag and drop
-        # (see http://doc.qt.nokia.com/4.6/model-view-dnd.html)
+        # see
+        # https://doc.qt.io/qt-4.8/model-view-programming.html#using-drag-and-drop-with-item-views
         self.setDragEnabled(True)
         self.setAcceptDrops(True)
         self.setAutoScroll(True)
@@ -53,17 +52,6 @@ class RepoTreeView(QTreeView):
         self.setEditTriggers(QAbstractItemView.DoubleClicked
                              | QAbstractItemView.EditKeyPressed)
         self.setSelectionBehavior(QAbstractItemView.SelectRows)
-        QShortcut('Return', self, self.showFirstTabOrOpen).setContext(
-                  Qt.WidgetShortcut)
-        QShortcut('Enter', self, self.showFirstTabOrOpen).setContext(
-                  Qt.WidgetShortcut)
-        QShortcut('Delete', self, self.removeSelected).setContext(
-                  Qt.WidgetShortcut)
-
-    def contextMenuEvent(self, event):
-        if not self.selitem:
-            return
-        self.menuRequested.emit(event.globalPos(), self.selitem)
 
     def dragEnterEvent(self, event):
         if event.source() is self:
@@ -148,6 +136,20 @@ class RepoTreeView(QTreeView):
         self.viewport().update()
         self.setAutoScroll(True)
 
+    def keyPressEvent(self, event):
+        if (event.key() in (Qt.Key_Enter, Qt.Key_Return)
+            and self.state() != QAbstractItemView.EditingState):
+            index = self.currentIndex()
+            if index.isValid():
+                self.openRequested.emit(index)
+                return
+        if event.key() == Qt.Key_Delete:
+            index = self.currentIndex()
+            if index.isValid():
+                self.removeRequested.emit(index)
+                return
+        super(RepoTreeView, self).keyPressEvent(event)
+
     def mouseMoveEvent(self, event):
         self.msg  = ''
         pos = event.pos()
@@ -159,7 +161,7 @@ class RepoTreeView(QTreeView):
 
         if event.buttons() == Qt.NoButton:
             # Bail out early to avoid tripping over this bug:
-            # http://bugreports.qt.nokia.com/browse/QTBUG-10180
+            # https://bugreports.qt.io/browse/QTBUG-10180
             return
         super(RepoTreeView, self).mouseMoveEvent(event)
 
@@ -168,61 +170,18 @@ class RepoTreeView(QTreeView):
             self.showMessage.emit('')
 
     def mouseDoubleClickEvent(self, event):
-        if self.selitem and self.selitem.internalPointer().isRepo():
-            self.showFirstTabOrOpen()
+        index = self.indexAt(event.pos())
+        if index.isValid() and index.internalPointer().isRepo():
+            self.openRequested.emit(index)
         else:
             # a double-click on non-repo rows opens an editor
             super(RepoTreeView, self).mouseDoubleClickEvent(event)
 
-    def selectionChanged(self, selected, deselected):
-        selection = self.selectedIndexes()
-        if len(selection) == 0:
-            self.selitem = None
-        else:
-            self.selitem = selection[0]
-
     def sizeHint(self):
         size = super(RepoTreeView, self).sizeHint()
         size.setWidth(QFontMetrics(self.font()).width('M') * 15)
         return size
 
-    def showFirstTabOrOpen(self):
-        'Enter or double click events, show existing or open a new repowidget'
-        if self.selitem and self.selitem.internalPointer().isRepo():
-            # We can only open mercurial repositories and subrepositories
-            repotype = self.selitem.internalPointer().repotype()
-            if repotype == 'hg':
-                root = self.selitem.internalPointer().rootpath()
-                self.openRepo.emit(hglib.tounicode(root), True)
-            else:
-                qtlib.WarningMsgBox(
-                    _('Unsupported repository type (%s)') % repotype,
-                    _('Cannot open non Mercurial repositories or '
-                      'subrepositories'),
-                    parent=self)
-
-    def removeSelected(self):
-        'remove selected repository'
-        s = self.selitem
-        if not s:
-            return
-        item = s.internalPointer()
-        if 'remove' not in item.menulist():  # check capability
-            return
-        if not item.okToDelete():
-            labels = [(QMessageBox.Yes, _('&Delete')),
-                      (QMessageBox.No, _('Cancel'))]
-            if not qtlib.QuestionMsgBox(_('Confirm Delete'),
-                                    _("Delete Group '%s' and all its entries?")%
-                                    item.name, labels=labels, parent=self):
-                return
-        m = self.model()
-        row = s.row()
-        parent = s.parent()
-        m.removeRows(row, 1, parent)
-        self.selectionChanged(None, None)
-        self.updateSettingsFile.emit()
-
 class RepoRegistryView(QDockWidget):
 
     showMessage = pyqtSignal(str)
@@ -257,10 +216,11 @@ class RepoRegistryView(QDockWidget):
         tv.setFirstColumnSpanned(0, QModelIndex(), True)
         tv.setColumnHidden(1, True)
 
+        tv.setContextMenuPolicy(Qt.CustomContextMenu)
+        tv.customContextMenuRequested.connect(self._onMenuRequested)
         tv.showMessage.connect(self.showMessage)
-        tv.menuRequested.connect(self.onMenuRequest)
-        tv.openRepo.connect(self.openRepo)
-        tv.updateSettingsFile.connect(self.updateSettingsFile)
+        tv.openRequested.connect(self._openRepoAt)
+        tv.removeRequested.connect(self._removeAt)
         tv.dropAccepted.connect(self.dropAccepted)
 
         self.createActions()
@@ -510,13 +470,16 @@ class RepoRegistryView(QDockWidget):
                 act.triggered.connect(cb)
             self.addAction(act)
 
-    def onMenuRequest(self, point, selitem):
-        menulist = selitem.internalPointer().menulist()
+    @pyqtSlot(QPoint)
+    def _onMenuRequested(self, pos):
+        index = self.tview.currentIndex()
+        if not index.isValid():
+            return
+        menulist = index.internalPointer().menulist()
         if not menulist:
             return
         self.addtomenu(self.contextmenu, menulist)
-        self.selitem = selitem
-        self.contextmenu.exec_(point)
+        self.contextmenu.popup(self.tview.viewport().mapToGlobal(pos))
 
     def addtomenu(self, menu, actlist):
         menu.clear()
@@ -533,17 +496,22 @@ class RepoRegistryView(QDockWidget):
     ## Menu action handlers
     #
 
+    def _currentRepoRoot(self):
+        model = self.tview.model()
+        index = self.tview.currentIndex()
+        return model.repoRoot(index)
+
     def cloneRepo(self):
-        root = self.selitem.internalPointer().rootpath()
-        self.cloneRepoRequested.emit(hglib.tounicode(root))
+        self.cloneRepoRequested.emit(self._currentRepoRoot())
 
     def explore(self):
-        root = self.selitem.internalPointer().rootpath()
-        qtlib.openlocalurl(root)
+        qtlib.openlocalurl(self._currentRepoRoot())
 
     def terminal(self):
-        repoitem = self.selitem.internalPointer()
-        qtlib.openshell(repoitem.rootpath(),
+        model = self.tview.model()
+        index = self.tview.currentIndex()
+        repoitem = index.internalPointer()
+        qtlib.openshell(hglib.fromunicode(model.repoRoot(index)),
                         hglib.fromunicode(repoitem.shortname()))
 
     def addNewRepo(self):
@@ -556,12 +524,12 @@ class RepoRegistryView(QDockWidget):
             m = self.tview.model()
             uroot = paths.find_root(unicode(path))
             if uroot and not m.isKnownRepoRoot(uroot, standalone=True):
-                index = m.addRepo(uroot, parent=self.selitem)
+                index = m.addRepo(uroot, parent=self.tview.currentIndex())
                 self._scanAddedRepo(index)
 
     def addSubrepo(self):
         'menu action handler for adding a new subrepository'
-        root = hglib.tounicode(self.selitem.internalPointer().rootpath())
+        root = self._currentRepoRoot()
         caption = _('Select an existing repository to add as a subrepo')
         FD = QFileDialog
         path = unicode(FD.getExistingDirectory(caption=caption,
@@ -605,7 +573,8 @@ class RepoRegistryView(QDockWidget):
 
                 # Is is already on the selected repository substate list?
                 try:
-                    repo = hg.repository(ui.ui(), hglib.fromunicode(root))
+                    repo = hg.repository(hglib.loadui(),
+                                         hglib.fromunicode(root))
                 except:
                     qtlib.WarningMsgBox(_('Cannot open repository'),
                         _('The selected repository:<br><br>%s<br><br>'
@@ -624,7 +593,7 @@ class RepoRegistryView(QDockWidget):
                     hasHgsub = os.path.exists(repo.wjoin('.hgsub'))
                     if hasHgsub:
                         try:
-                            fsub = repo.wopener('.hgsub', 'r')
+                            fsub = repo.wvfs('.hgsub', 'r')
                             lines = fsub.readlines()
                             fsub.close()
                         except:
@@ -663,11 +632,12 @@ class RepoRegistryView(QDockWidget):
 
                     # and update the .hgsub file
                     try:
-                        fsub = repo.wopener('.hgsub', 'w')
+                        fsub = repo.wvfs('.hgsub', 'w')
                         fsub.write(linesep.join(lines) + linesep)
                         fsub.close()
                         if not hasHgsub:
-                            commands.add(ui.ui(), repo, repo.wjoin('.hgsub'))
+                            commands.add(hglib.loadui(),
+                                         repo, repo.wjoin('.hgsub'))
                         qtlib.InfoMsgBox(
                             _('Subrepo added to .hgsub file'),
                             _('The selected subrepo:<br><br><i>%s</i><br><br>'
@@ -687,8 +657,9 @@ class RepoRegistryView(QDockWidget):
     def removeSubrepo(self):
         'menu action handler for removing an existing subrepository'
         model = self.tview.model()
-        path = model.repoRoot(self.selitem)
-        root = model.repoRoot(self.selitem.parent())
+        index = self.tview.currentIndex()
+        path = model.repoRoot(index)
+        root = model.repoRoot(index.parent())
         relsubpath = os.path.normcase(os.path.normpath(path[1+len(root):]))
         hgsubfilename = os.path.join(root, '.hgsub')
 
@@ -743,38 +714,56 @@ class RepoRegistryView(QDockWidget):
                 parent=self)
 
     def startSettings(self):
-        root = self.selitem.internalPointer().rootpath()
+        root = hglib.fromunicode(self._currentRepoRoot())
         sd = settings.SettingsDialog(configrepo=True, focus='web.name',
                                      parent=self, root=root)
         sd.finished.connect(sd.deleteLater)
         sd.exec_()
 
     def openAll(self):
-        for root in self.selitem.internalPointer().childRoots():
-            self.openRepo.emit(hglib.tounicode(root), False)
+        index = self.tview.currentIndex()
+        for root in index.internalPointer().childRoots():
+            self.openRepo.emit(root, False)
 
     def open(self, root=None):
         'open context menu action, open repowidget unconditionally'
         if not root:
-            root = self.selitem.internalPointer().rootpath()
-            repotype = self.selitem.internalPointer().repotype()
+            model = self.tview.model()
+            index = self.tview.currentIndex()
+            root = model.repoRoot(index)
+            repotype = index.internalPointer().repotype()
         else:
-            root = hglib.fromunicode(root)
             if os.path.exists(os.path.join(root, '.hg')):
                 repotype = 'hg'
             else:
                 repotype = 'unknown'
         if repotype == 'hg':
-            self.openRepo.emit(hglib.tounicode(root), False)
+            self.openRepo.emit(root, False)
         else:
             qtlib.WarningMsgBox(
                 _('Unsupported repository type (%s)') % repotype,
                 _('Cannot open non Mercurial repositories or subrepositories'),
                 parent=self)
 
+    @pyqtSlot(QModelIndex)
+    def _openRepoAt(self, index):
+        model = self.tview.model()
+        root = model.repoRoot(index)
+        if root:
+            # We can only open mercurial repositories and subrepositories
+            repotype = index.internalPointer().repotype()
+            if repotype == 'hg':
+                self.openRepo.emit(root, True)
+            else:
+                qtlib.WarningMsgBox(
+                    _('Unsupported repository type (%s)') % repotype,
+                    _('Cannot open non Mercurial repositories or '
+                      'subrepositories'),
+                    parent=self)
+
     def copyPath(self):
         clip = QApplication.clipboard()
-        clip.setText(hglib.tounicode(self.selitem.internalPointer().rootpath()))
+        clip.setText(self._currentRepoRoot())
 
     def startRename(self):
         self.tview.edit(self.tview.currentIndex())
@@ -783,29 +772,47 @@ class RepoRegistryView(QDockWidget):
         self.tview.model().addGroup(_('New Group'))
 
     def removeSelected(self):
-        ip = self.selitem.internalPointer()
-        if ip.isRepo():
-            root = ip.rootpath()
-        else:
-            root = None
+        root = self._currentRepoRoot()
+        self._removeAt(self.tview.currentIndex())
+        if root:
+            self.removeRepo.emit(root)
 
-        self.tview.removeSelected()
-
-        if root is not None:
-            self.removeRepo.emit(hglib.tounicode(root))
+    @pyqtSlot(QModelIndex)
+    def _removeAt(self, index):
+        item = index.internalPointer()
+        if 'remove' not in item.menulist():  # check capability
+            return
+        if not item.okToDelete():
+            labels = [(QMessageBox.Yes, _('&Delete')),
+                      (QMessageBox.No, _('Cancel'))]
+            if not qtlib.QuestionMsgBox(_('Confirm Delete'),
+                                        _("Delete Group '%s' and all its "
+                                          "entries?") % item.name,
+                                        labels=labels, parent=self):
+                return
+        m = self.tview.model()
+        m.removeRows(index.row(), 1, index.parent())
+        self.updateSettingsFile()
 
     def sortbyname(self):
-        childs = self.selitem.internalPointer().childs
+        index = self.tview.currentIndex()
+        childs = index.internalPointer().childs
         self.tview.model().sortchilds(childs, lambda x: x.shortname().lower())
 
     def sortbypath(self):
-        childs = self.selitem.internalPointer().childs
-        self.tview.model().sortchilds(
-            childs, lambda x: os.path.normcase(util.normpath(x.rootpath())))
+        index = self.tview.currentIndex()
+        childs = index.internalPointer().childs
+        def keyfunc(x):
+            l = hglib.fromunicode(x.rootpath())
+            return os.path.normcase(util.normpath(l))
+        self.tview.model().sortchilds(childs, keyfunc)
 
     def sortbyhgsub(self):
-        ip = self.selitem.internalPointer()
-        repo = hg.repository(ui.ui(), ip.rootpath())
+        model = self.tview.model()
+        index = self.tview.currentIndex()
+        ip = index.internalPointer()
+        repo = hg.repository(hglib.loadui(),
+                             hglib.fromunicode(model.repoRoot(index)))
         ctx = repo['.']
         wfile = '.hgsub'
         if wfile not in ctx:
@@ -816,9 +823,10 @@ class RepoRegistryView(QDockWidget):
         abspath = lambda x: util.normpath(repo.wjoin(x))
         hgsuborder = [abspath(getsubpath(x)) for x in data]
         def keyfunc(x):
+            l = hglib.fromunicode(x.rootpath())
             try:
-                return hgsuborder.index(util.normpath(x.rootpath()))
-            except:
+                return hgsuborder.index(util.normpath(l))
+            except ValueError:
                 # If an item is not found, place it at the top
                 return 0
         self.tview.model().sortchilds(ip.childs, keyfunc)
@@ -845,6 +853,7 @@ class RepoRegistryView(QDockWidget):
 
     @pyqtSlot(str)
     def scanRepo(self, uroot):
+        uroot = unicode(uroot)
         m = self.tview.model()
         index = m.indexFromRepoRoot(uroot)
         if index.isValid():
