@@ -8,8 +8,6 @@
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
-from mercurial import util
-
 from tortoisehg.util import hglib
 from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import cmdcore, qtlib
@@ -29,17 +27,15 @@ class SignDialog(QDialog):
         base = QVBoxLayout()
         base.setSpacing(0)
         base.setContentsMargins(*(0,)*4)
-        base.setSizeConstraint(QLayout.SetFixedSize)
+        base.setSizeConstraint(QLayout.SetMinAndMaxSize)
         self.setLayout(base)
 
-        box = QVBoxLayout()
-        box.setSpacing(8)
-        box.setContentsMargins(*(8,)*4)
-        self.layout().addLayout(box)
-
         ## main layout grid
+        formwidget = QWidget(self)
+        formwidget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         form = QFormLayout(fieldGrowthPolicy=QFormLayout.AllNonFixedFieldsGrow)
-        box.addLayout(form)
+        formwidget.setLayout(form)
+        base.addWidget(formwidget)
 
         repo = repoagent.rawRepo()
         form.addRow(_('Revision:'), QLabel('%s (%s)' % (rev, repo[rev])))
@@ -51,11 +47,9 @@ class SignDialog(QDialog):
         ### options
         expander = qtlib.ExpanderLabel(_('Options'), False)
         expander.expanded.connect(self.show_options)
-        box.addWidget(expander)
-
         optbox = QVBoxLayout()
         optbox.setSpacing(6)
-        box.addLayout(optbox)
+        form.addRow(expander, optbox)
 
         hbox = QHBoxLayout()
         hbox.setSpacing(0)
@@ -87,7 +81,7 @@ class SignDialog(QDialog):
         self.signBtn = bbox.addButton(_('&Sign'), BB.ActionRole)
         bbox.addButton(BB.Close)
         bbox.rejected.connect(self.reject)
-        box.addWidget(bbox)
+        form.addRow(bbox)
 
         self.signBtn.clicked.connect(self.onSign)
 
@@ -124,11 +118,10 @@ class SignDialog(QDialog):
             self.customMessageToggle(False)
         self.keyLineEdit.setFocus()
 
-        expanded = util.any([self.replaceCheckBox.isChecked(),
-                  self.localCheckBox.isChecked(),
-                  self.nocommitCheckBox.isChecked(),
-                  self.customCheckBox.isChecked()
-                  ])
+        expanded = any([self.replaceCheckBox.isChecked(),
+                        self.localCheckBox.isChecked(),
+                        self.nocommitCheckBox.isChecked(),
+                        self.customCheckBox.isChecked()])
         expander.set_expanded(expanded)
         self.show_options(expanded)
 
@@ -188,8 +181,8 @@ class SignDialog(QDialog):
             self.customTextLineEdit.setFocus()
 
     def set_status(self, text, icon=None):
-        self.status.setShown(True)
-        self.sep.setShown(True)
+        self.status.setVisible(True)
+        self.sep.setVisible(True)
         self.status.set_status(text, icon)
 
     def clear_status(self):
