@@ -7,7 +7,7 @@
 
 import os
 
-from mercurial import hg, util, error, context, merge, scmutil
+from mercurial import hg, util, error, context, scmutil
 
 from tortoisehg.util import hglib
 from tortoisehg.util.i18n import _
@@ -393,7 +393,7 @@ class StatusWidget(QWidget):
             self.updateModel(self.refthread.wctx, self.refthread.wstatus,
                              self.refthread.patchecked)
         self.refthread = None
-        if len(self.repo.parents()) > 1:
+        if len(self.repo[None].parents()) > 1:
             # nuke partial selections if wctx has a merge in-progress
             self.partials = {}
         match = self.le.text()
@@ -421,7 +421,7 @@ class StatusWidget(QWidget):
                 qtlib.WarningMsgBox(_('No appropriate files'),
                                     _('No files found for this operation'),
                                     parent=self)
-        ms = merge.mergestate(self.repo)
+        ms = hglib.readmergestate(self.repo)
         tm = WctxModel(self._repoagent, wctx, wstatus, ms, self.pctx,
                        self.savechecks, self.opts, checked, self,
                        checkable=self.checkable, defcheck=self.defcheck)
@@ -632,7 +632,6 @@ class StatusThread(QThread):
         self.patchecked = {}
 
     def run(self):
-        self.repo.dirstate.invalidate()
         extract = lambda x, y: dict(zip(x, map(y.get, x)))
         stopts = extract(('unknown', 'ignored', 'clean'), self.opts)
         patchecked = {}
@@ -799,9 +798,9 @@ class WctxModel(QAbstractTableModel):
         if not index.isValid():
             return QVariant()
 
-        path, status, mst, upath, ext, sz = self.rows[index.row()]
         if index.column() == COL_PATH:
             if role == Qt.CheckStateRole and self.checkable:
+                path = self.rows[index.row()][0]
                 if path in self.partials:
                     changes = self.partials[path]
                     if changes.excludecount == 0:
@@ -821,11 +820,13 @@ class WctxModel(QAbstractTableModel):
         elif role == Qt.DisplayRole:
             return QVariant(self.rows[index.row()][index.column()])
         elif role == Qt.TextColorRole:
+            path, status, mst, upath, ext, sz = self.rows[index.row()]
             if mst:
                 return _colors.get(mst.lower(), QColor('black'))
             else:
                 return _colors.get(status, QColor('black'))
         elif role == Qt.ToolTipRole:
+            path, status, mst, upath, ext, sz = self.rows[index.row()]
             return QVariant(statusMessage(status, mst, upath))
         '''
         elif role == Qt.DecorationRole and index.column() == COL_STATUS:
@@ -890,6 +891,11 @@ class WctxModel(QAbstractTableModel):
                 return True
         return False
 
+    @util.propertycache
+    def workingContext(self):
+        repo = self._repoagent.rawRepo()
+        return repo[None]
+
     def fileData(self, index):
         """Returns the displayable file data at the given index"""
         repo = self._repoagent.rawRepo()
@@ -897,7 +903,7 @@ class WctxModel(QAbstractTableModel):
             return filedata.createNullData(repo)
         path, status, mst, upath, ext, sz = self.rows[index.row()]
         wfile = util.pconvert(path)
-        ctx = repo[None]
+        ctx = self.workingContext
         pctx = self._pctx and self._pctx.p1() or ctx.p1()
         if status == 'S':
             return filedata.createSubrepoData(ctx, pctx, wfile)
