@@ -12,7 +12,7 @@ import tempfile
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
-from mercurial import hg, util, scmutil, httpconnection
+from mercurial import hg, util, httpconnection
 
 from tortoisehg.util import hglib, paths, wconfig
 from tortoisehg.util.i18n import _
@@ -173,6 +173,12 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.urlentry.textChanged.connect(self.urlChanged)
         self.urlentry.returnPressed.connect(self.saveclicked)
         tbar.addWidget(self.urlentry)
+        tbar.addWidget(qtlib.Spacer(2, 2))
+
+        self.browsebutton = QPushButton(_('Browse...'))
+        self.browsebutton.setAutoDefault(False)
+        self.browsebutton.clicked.connect(self._browseUrl)
+        tbar.addWidget(self.browsebutton)
 
         # even though currentRowChanged fires pathSelected, clicked signal is
         # also connected to it. otherwise urlentry won't be updated when the
@@ -300,7 +306,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     def reload(self):
         # Refresh configured paths
         self.paths = {}
-        fn = self.repo.join('hgrc')
+        fn = self.repo.vfs.join('hgrc')
         fn, cfg = hgrcutil.loadIniFile([fn], self)
         if 'paths' in cfg:
             for alias in cfg['paths']:
@@ -528,6 +534,14 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
             self.curalias = hglib.fromunicode(dlg.aliasentry.text())
             self.reload()
 
+    @pyqtSlot()
+    def _browseUrl(self):
+        FD = QFileDialog
+        caption = _("Select repository")
+        path = FD.getExistingDirectory(self, caption, self.urlentry.text())
+        if path:
+            self.urlentry.setText(QDir.toNativeSeparators(path))
+
     def secureclicked(self):
         if not parseurl(self.currentUrl()).host:
             qtlib.WarningMsgBox(_('No host specified'),
@@ -743,7 +757,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
         self.pullCompleted.emit()
         # handle file conflicts during rebase
         if self.cachedpp in ('rebase', 'updateorrebase'):
-            if os.path.exists(self.repo.join('rebasestate')):
+            if os.path.exists(self.repo.vfs.join('rebasestate')):
                 dlg = rebase.RebaseDialog(self._repoagent, self)
                 dlg.exec_()
                 return
@@ -974,7 +988,7 @@ class SyncWidget(QWidget, qtlib.TaskWidget):
     @pyqtSlot(str)
     def removeAlias(self, alias):
         alias = hglib.fromunicode(alias)
-        fn = self.repo.join('hgrc')
+        fn = self.repo.vfs.join('hgrc')
         fn, cfg = hgrcutil.loadIniFile([fn], self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to remove URL'),
@@ -1076,7 +1090,7 @@ class PostPullDialog(QDialog):
                     if chk.isChecked()).next()
 
     def accept(self):
-        path = self.repo.join('hgrc')
+        path = self.repo.vfs.join('hgrc')
         fn, cfg = hgrcutil.loadIniFile([path], self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to save post pull operation'),
@@ -1160,7 +1174,7 @@ class SaveDialog(QDialog):
         self._updateUi()
 
     def savePath(self, repo, alias, path, confirm=True):
-        fn = repo.join('hgrc')
+        fn = repo.vfs.join('hgrc')
         fn, cfg = hgrcutil.loadIniFile([fn], self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to save an URL'),
@@ -1291,7 +1305,11 @@ class SecureDialog(QDialog):
         self.insecureradio = QRadioButton(
             _('No host validation, but still encrypted (bad)'))
         hbox = QHBoxLayout()
-        fprint = repo.ui.config('hostfingerprints', u.host, '')
+        fprint = repo.ui.config('hostsecurity', u.host + ':fingerprints', '')
+        if not fprint:
+            fprint = repo.ui.config('hostfingerprints', u.host, '')
+            if fprint:
+                fprint = 'sha1:' + fprint
         self.fprintentry = le = QLineEdit(fprint)
         self.fprintradio.toggled.connect(self.fprintentry.setEnabled)
         self.fprintentry.setEnabled(False)
@@ -1300,7 +1318,6 @@ class SecureDialog(QDialog):
         hbox.addWidget(le)
         self._querybutton = qb = QPushButton(_('Query'))
         qb.clicked.connect(self._queryFingerprint)
-        self.fprintradio.toggled.connect(self._updateUi)
         hbox.addWidget(qb)
         vbox.addWidget(self.cacertradio)
         vbox.addWidget(self.fprintradio)
@@ -1313,6 +1330,21 @@ class SecureDialog(QDialog):
         elif repo.ui.config('insecurehosts', u.host):
             self.insecureradio.setChecked(True)
 
+        self.fprintradio.toggled.connect(self._updateUi)
+        self.insecureradio.toggled.connect(self._updateUi)
+
+        self._protocolcombo = e = QComboBox(self)
+        e.addItem(_('<unspecified>'), '')
+        e.addItem(_('TLS 1.0'), 'tls1.0')
+        e.addItem(_('TLS 1.1'), 'tls1.1')
+        e.addItem(_('TLS 1.2'), 'tls1.2')
+        protocol = repo.ui.config('hostsecurity', '%s:minimumprotocol' % u.host)
+        e.setCurrentIndex(e.findData(hglib.tounicode(protocol or '')))
+        hbox = QHBoxLayout()
+        hbox.addWidget(QLabel(_('Minimum Protocol')))
+        hbox.addWidget(self._protocolcombo)
+        vbox.addLayout(hbox)
+
         self._authentries = {}  # key: QLineEdit
         authbox = QGroupBox(_('User Authentication'))
         form = QFormLayout()
@@ -1374,7 +1406,8 @@ are expanded in the filename.'''))
 
     @pyqtSlot()
     def _queryFingerprint(self):
-        cmdline = hglib.buildcmdargs('debuggethostfingerprint', self._url)
+        cmdline = hglib.buildcmdargs('debuggethostfingerprint', self._url,
+                                     insecure=True)
         self._querysess = sess = self._repoagent.runCommand(cmdline, self)
         sess.setCaptureOutput(True)
         sess.commandFinished.connect(self._onQueryFingerprintFinished)
@@ -1412,7 +1445,7 @@ are expanded in the filename.'''))
             e.setText(n)
 
     def accept(self):
-        path = scmutil.userrcpath()
+        path = hglib.userrcpath()
         fn, cfg = hgrcutil.loadIniFile(path, self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to save authentication'),
@@ -1436,9 +1469,13 @@ are expanded in the filename.'''))
         else:
             fprint = None
             insecure = '1'
-        setorclear('hostfingerprints', self.host, fprint)
+        setorclear('hostsecurity', '%s:fingerprints' % self.host, fprint)
         setorclear('insecurehosts', self.host, insecure)
 
+        e = self._protocolcombo
+        protocol = hglib.fromunicode(e.itemData(e.currentIndex()).toString())
+        setorclear('hostsecurity', '%s:minimumprotocol' % self.host, protocol)
+
         cfg.set('auth', self.alias+'.prefix', self.host)
         for k in ['username', 'password', 'key', 'cert']:
             setorclear('auth', '%s.%s' % (self.alias, k),
@@ -1457,6 +1494,7 @@ are expanded in the filename.'''))
     def _updateUi(self):
         self._querybutton.setEnabled(self.fprintradio.isChecked()
                                      and self._querysess.isFinished())
+        self._protocolcombo.setEnabled(not self.insecureradio.isChecked())
 
 
 class PathsTree(QTreeView):
