@@ -18,8 +18,7 @@ import time
 from PyQt4.QtCore import *
 
 from mercurial import hg, error, bundlerepo, extensions, filemerge, node
-from mercurial import localrepo, merge, subrepo
-from mercurial import ui as uimod
+from mercurial import localrepo, subrepo
 from hgext import mq
 
 from tortoisehg.util import hglib, paths
@@ -37,7 +36,7 @@ def repository(_ui=None, path=''):
     is obtained using mercurial.hg.repository()'''
     if path not in _repocache:
         if _ui is None:
-            _ui = uimod.ui()
+            _ui = hglib.loadui()
         try:
             repo = hg.repository(_ui, path)
             repo = repo.unfiltered()
@@ -67,6 +66,10 @@ WorkingParentChanged = 0x2
 WorkingBranchChanged = 0x4
 WorkingStateChanged = 0x8  # internal flag to invalidate dirstate cache
 
+_PollDeferred = 0x1  # flag to defer polling
+_PollFsChangesPending = 0x2
+_PollStatusPending = 0x4
+
 
 class RepoWatcher(QObject):
     """Notify changes of repository by optionally monitoring filesystem"""
@@ -80,9 +83,10 @@ class RepoWatcher(QObject):
         self._repo = repo
         self._ui = repo.ui
         self._fswatcher = None
+        self._deferredpoll = 0  # _Poll* flags
         self._filesmap = {}  # path: (flag, watched)
         self._datamap = {}  # readmeth: (flag, dep-path)
-        self._lastmtimes = {}  # path: mtime
+        self._laststats = {}  # path: (size, ctime, mtime)
         self._lastdata = {}  # readmeth: content
         self._fixState()
         self._uimtime = time.time()
@@ -91,15 +95,19 @@ class RepoWatcher(QObject):
         """Start filesystem monitoring to notify changes automatically"""
         if not self._fswatcher:
             self._fswatcher = QFileSystemWatcher(self)
-            self._fswatcher.directoryChanged.connect(self._pollChanges)
-            self._fswatcher.fileChanged.connect(self._pollChanges)
+            self._fswatcher.directoryChanged.connect(self._onFsChanged)
+            self._fswatcher.fileChanged.connect(self._onFsChanged)
         self._fswatcher.addPath(hglib.tounicode(self._repo.path))
         self._fswatcher.addPath(hglib.tounicode(self._repo.spath))
         self._addMissingPaths()
         self._fswatcher.blockSignals(False)
 
     def stopMonitoring(self):
-        """Stop filesystem monitoring by removing all watched paths"""
+        """Stop filesystem monitoring by removing all watched paths
+
+        This will release OS resources held by filesystem watcher, so good
+        for disabling change notification for a long time.
+        """
         if not self._fswatcher:
             return
         self._fswatcher.blockSignals(True)  # ignore pending events
@@ -124,19 +132,44 @@ class RepoWatcher(QObject):
             return False
         return not self._fswatcher.signalsBlocked()
 
+    def resumeStatusPolling(self):
+        """Execute deferred status checks to emit notification signals"""
+        self._deferredpoll &= ~_PollDeferred
+        if self._deferredpoll & _PollFsChangesPending:
+            self._pollFsChanges()
+            self._deferredpoll &= ~(_PollFsChangesPending | _PollStatusPending)
+        if self._deferredpoll & _PollStatusPending:
+            self._pollStatus()
+            self._deferredpoll &= ~_PollStatusPending
+
+    def suspendStatusPolling(self):
+        """Defer status checks until resumed
+
+        Resuming from suspended state should be cheaper, but no OS resources
+        will be released. This is good for short-time suspend.
+        """
+        self._deferredpoll |= _PollDeferred
+
     @pyqtSlot()
-    def _pollChanges(self):
+    def _onFsChanged(self):
+        if self._deferredpoll:
+            self._ui.debug('filesystem change detected, but poll deferred\n')
+            self._deferredpoll |= _PollFsChangesPending
+            return
+        self._pollFsChanges()
+
+    def _pollFsChanges(self):
         '''Catch writes or deletions of files, or writes to .hg/ folder,
         most importantly lock files'''
-        self.pollStatus()
-        # filesystem monitor may be stopped inside pollStatus()
+        self._pollStatus()
+        # filesystem monitor may be stopped inside _pollStatus()
         if self.isMonitoring():
             self._addMissingPaths()
 
     def _addMissingPaths(self):
         'Add files to watcher that may have been added or replaced'
         existing = [f for f, (_flag, watched) in self._filesmap.iteritems()
-                    if watched and f in self._lastmtimes]
+                    if watched and f in self._laststats]
         files = [unicode(f) for f in self._fswatcher.files()]
         for f in existing:
             if hglib.tounicode(f) not in files:
@@ -148,10 +181,17 @@ class RepoWatcher(QObject):
                 self._fswatcher.addPath(hglib.tounicode(f))
 
     def clearStatus(self):
-        self._lastmtimes.clear()
+        self._laststats.clear()
         self._lastdata.clear()
 
     def pollStatus(self):
+        if self._deferredpoll:
+            self._ui.debug('poll request deferred\n')
+            self._deferredpoll |= _PollStatusPending
+            return
+        self._pollStatus()
+
+    def _pollStatus(self):
         if not os.path.exists(self._repo.path):
             self._ui.debug('repository destroyed: %s\n' % self._repo.root)
             self.repositoryDestroyed.emit()
@@ -159,12 +199,12 @@ class RepoWatcher(QObject):
         if self._locked():
             self._ui.debug('locked, aborting\n')
             return
-        curmtimes, curdata = self._readState()
-        changeflags = self._calculateChangeFlags(curmtimes, curdata)
+        curstats, curdata = self._readState()
+        changeflags = self._calculateChangeFlags(curstats, curdata)
         if self._locked():
             self._ui.debug('lock still held - ignoring for now\n')
             return
-        self._lastmtimes = curmtimes
+        self._laststats = curstats
         self._lastdata = curdata
         if changeflags:
             self._ui.debug('change found (flags = 0x%x)\n' % changeflags)
@@ -173,9 +213,9 @@ class RepoWatcher(QObject):
         self._checkuimtime()
 
     def _locked(self):
-        if os.path.lexists(self._repo.join('wlock')):
+        if os.path.lexists(self._repo.vfs.join('wlock')):
             return True
-        if os.path.lexists(self._repo.sjoin('lock')):
+        if os.path.lexists(self._repo.svfs.join('lock')):
             return True
         return False
 
@@ -184,22 +224,22 @@ class RepoWatcher(QObject):
         repo = self._repo
         q = getattr(repo, 'mq', None)
         newfilesmap = {
-            repo.join('bookmarks'): (LogChanged, False),
-            repo.join('bookmarks.current'): (LogChanged, False),
-            repo.join('branch'): (0, False),
-            repo.join('dirstate'): (WorkingStateChanged, False),
-            repo.join('localtags'): (LogChanged, False),
-            repo.sjoin('00changelog.i'): (LogChanged, False),
-            repo.sjoin('obsstore'): (LogChanged, False),
-            repo.sjoin('phaseroots'): (LogChanged, False),
+            repo.vfs.join('bookmarks'): (LogChanged, False),
+            repo.vfs.join('bookmarks.current'): (LogChanged, False),
+            repo.vfs.join('branch'): (0, False),
+            repo.vfs.join('dirstate'): (WorkingStateChanged, False),
+            repo.vfs.join('localtags'): (LogChanged, False),
+            repo.svfs.join('00changelog.i'): (LogChanged, False),
+            repo.svfs.join('obsstore'): (LogChanged, False),
+            repo.svfs.join('phaseroots'): (LogChanged, False),
             }
         if q:
             newfilesmap.update({
                 q.join('guards'): (LogChanged, True),
                 q.join('series'): (LogChanged, True),
                 q.join('status'): (LogChanged, True),
-                repo.join('patches.queue'): (LogChanged, True),
-                repo.join('patches.queues'): (LogChanged, True),
+                repo.vfs.join('patches.queue'): (LogChanged, True),
+                repo.vfs.join('patches.queues'): (LogChanged, True),
                 })
         newpaths = set(newfilesmap) - set(self._filesmap)
         if not newpaths:
@@ -207,22 +247,25 @@ class RepoWatcher(QObject):
         self._filesmap = newfilesmap
         self._datamap = {
             RepoWatcher._readbranch: (WorkingBranchChanged,
-                                      repo.join('branch')),
+                                      repo.vfs.join('branch')),
             RepoWatcher._readparents: (WorkingParentChanged,
-                                       repo.join('dirstate')),
+                                       repo.vfs.join('dirstate')),
             }
-        newmtimes, newdata = self._readState(newpaths)
-        self._lastmtimes.update(newmtimes)
+        newstats, newdata = self._readState(newpaths)
+        self._laststats.update(newstats)
         self._lastdata.update(newdata)
 
     def _readState(self, targetpaths=None):
         if targetpaths is None:
             targetpaths = self._filesmap
 
-        curmtimes = {}
+        curstats = {}
         for path in targetpaths:
             try:
-                curmtimes[path] = os.path.getmtime(path)
+                # see mercurial.util.filestat for details what attributes
+                # are needed an how ambiguity is resolved
+                st = os.stat(path)
+                curstats[path] = (st.st_size, st.st_ctime, st.st_mtime)
             except EnvironmentError:
                 pass
 
@@ -230,9 +273,9 @@ class RepoWatcher(QObject):
         for readmeth, (_flag, path) in self._datamap.iteritems():
             if path not in targetpaths:
                 continue
-            last = self._lastmtimes.get(path, -1)
-            cur = curmtimes.get(path, -1)
-            if last != cur:  # mtime can go back on rollback
+            last = self._laststats.get(path, -1)
+            cur = curstats.get(path, -1)
+            if last != cur:
                 try:
                     curdata[readmeth] = readmeth(self)
                 except EnvironmentError:
@@ -240,15 +283,15 @@ class RepoWatcher(QObject):
             elif cur >= 0 and readmeth in self._lastdata:
                 curdata[readmeth] = self._lastdata[readmeth]
 
-        return curmtimes, curdata
+        return curstats, curdata
 
-    def _calculateChangeFlags(self, curmtimes, curdata):
+    def _calculateChangeFlags(self, curstats, curdata):
         changeflags = 0
         for path, (flag, _watched) in self._filesmap.iteritems():
-            last = self._lastmtimes.get(path, -1)
-            cur = curmtimes.get(path, -1)
-            if last != cur:  # mtime can go back on rollback
-                self._ui.debug(' mtime: %s (%r -> %r)\n' % (path, last, cur))
+            last = self._laststats.get(path, -1)
+            cur = curstats.get(path, -1)
+            if last != cur:
+                self._ui.debug(' stat: %s (%r -> %r)\n' % (path, last, cur))
                 changeflags |= flag
         for readmeth, (flag, _path) in self._datamap.iteritems():
             last = self._lastdata.get(readmeth)
@@ -260,10 +303,10 @@ class RepoWatcher(QObject):
         return changeflags
 
     def _readparents(self):
-        return self._repo.opener('dirstate').read(40)
+        return self._repo.vfs('dirstate').read(40)
 
     def _readbranch(self):
-        return self._repo.opener('branch').read()
+        return self._repo.vfs('branch').read()
 
     def _checkuimtime(self):
         'Check for modified config files, or a new .hg/hgrc file'
@@ -323,20 +366,15 @@ class RepoAgent(QObject):
         self._subrepoagents = {}  # path: agent
 
     def startMonitoringIfEnabled(self):
-        """Start filesystem monitoring on repository open by RepoManager or
-        running command finished"""
+        """Start filesystem monitoring on repository open by RepoManager"""
         repo = self._repo
         ui = repo.ui
         monitorrepo = repo.ui.config('tortoisehg', 'monitorrepo', 'localonly')
         if monitorrepo == 'never':
             ui.debug('watching of F/S events is disabled by configuration\n')
-        elif self._overlayurl:
-            ui.debug('not watching F/S events for overlay repository\n')
         elif (monitorrepo == 'localonly'
               and not paths.is_on_fixed_drive(repo.path)):
             ui.debug('not watching F/S events for network drive\n')
-        elif self.isBusy():
-            ui.debug('not watching F/S events while busy\n')
         else:
             self._watcher.startMonitoring()
 
@@ -356,9 +394,7 @@ class RepoAgent(QObject):
             self.serviceStopped.emit()
 
     def suspendMonitoring(self):
-        """Stop filesystem monitoring temporarily; may be resumed when command
-        finished or overlay repository changed"""
-        # no "suspended" status until we really need it
+        """Stop filesystem monitoring and release OS resources"""
         self._watcher.stopMonitoring()
 
     def resumeMonitoring(self):
@@ -416,7 +452,7 @@ class RepoAgent(QObject):
         repo = repo.filtered('visible')
         self._changeRepo(_filteredrepo(repo, self.hiddenRevsIncluded()))
         self._overlayurl = url
-        self.suspendMonitoring()
+        self._watcher.suspendStatusPolling()
         self._flushRepositoryChanged()
 
     def clearOverlay(self):
@@ -426,7 +462,8 @@ class RepoAgent(QObject):
         repo.thginvalidate()  # take changes during overlaid
         self._changeRepo(_filteredrepo(repo, self.hiddenRevsIncluded()))
         self._overlayurl = ''
-        self.resumeMonitoring()
+        self._watcher.resumeStatusPolling()
+        self._flushRepositoryChanged()
 
     def _changeRepo(self, repo):
         # bundle/union repo will append temporary revisions to changelog
@@ -449,9 +486,8 @@ class RepoAgent(QObject):
         self._watcher.clearStatus()
 
     def pollStatus(self):
-        """Force checking changes to emit corresponding signals"""
-        if self._cmdagent.isBusy():
-            return  # delayed until _onBusyChanged(False)
+        """Force checking changes to emit corresponding signals; this will be
+        deferred if command is running"""
         self._watcher.pollStatus()
         self._flushRepositoryChanged()
 
@@ -490,9 +526,14 @@ class RepoAgent(QObject):
     @pyqtSlot(bool)
     def _onBusyChanged(self, busy):
         if busy:
-            self.suspendMonitoring()
+            self._watcher.suspendStatusPolling()
         else:
-            self.resumeMonitoring()
+            self._watcher.resumeStatusPolling()
+            if not self._watcher.isMonitoring():
+                # detect changes made by the last command even if monitoring
+                # is disabled
+                self._watcher.pollStatus()
+            self._flushRepositoryChanged()
         self.busyChanged.emit(busy)
 
     def runCommand(self, cmdline, uihandler=None, overlay=True):
@@ -759,7 +800,7 @@ def _extendrepo(repo):
             for line in cfg._source.values():
                 f = line.rsplit(':', 1)[0]
                 files.add(f)
-            files.add(self.join('hgrc'))
+            files.add(self.vfs.join('hgrc'))
             return files
 
         @localrepo.unfilteredpropertycache
@@ -841,7 +882,7 @@ def _extendrepo(repo):
             return tag in self._thgmqpatchnames
 
         def thgshelves(self):
-            self.shelfdir = sdir = self.join('shelves')
+            self.shelfdir = sdir = self.vfs.join('shelves')
             if os.path.isdir(sdir):
                 def getModificationTime(x):
                     try:
@@ -862,7 +903,8 @@ def _extendrepo(repo):
 
         def thginvalidate(self):
             'Should be called when mtime of repo store/dirstate are changed'
-            self.dirstate.invalidate()
+            self.invalidatedirstate()
+
             if not isinstance(repo, bundlerepo.bundlerepository):
                 self.invalidate()
             # mq.queue.invalidate does not handle queue changes, so force
@@ -876,8 +918,8 @@ def _extendrepo(repo):
         def invalidateui(self):
             'Should be called when mtime of ui files are changed'
             origui = self.ui
-            self.ui = uimod.ui()
-            self.ui.readconfig(self.join('hgrc'))
+            self.ui = hglib.loadui()
+            self.ui.readconfig(self.vfs.join('hgrc'))
             hglib.copydynamicconfig(origui, self.ui)
             for a in _uiprops:
                 if localrepo.hasunfilteredcache(self, a):
@@ -886,7 +928,7 @@ def _extendrepo(repo):
         def thgbackup(self, path):
             'Make a backup of the given file in the repository "trashcan"'
             # The backup name will be the same as the orginal file plus '.bak'
-            trashcan = self.join('Trashcan')
+            trashcan = self.vfs.join('Trashcan')
             if not os.path.isdir(trashcan):
                 os.mkdir(trashcan)
             if not os.path.exists(path):
@@ -1023,7 +1065,7 @@ def genPatchContext(repo, patchpath, rev=None):
     return ctx
 
 def recursiveMergeStatus(repo):
-    ms = merge.mergestate(repo)
+    ms = hglib.readmergestate(repo)
     for wfile in ms:
         yield repo.root, wfile, ms[wfile]
     try:
@@ -1048,7 +1090,8 @@ def relatedRepositories(repoid):
         for e in repotreemodel.iterRepoItemFromXml(f):
             if e.basenode() == repoid:
                 # TODO: both in unicode because this is Qt-layer function?
-                yield e.rootpath(), hglib.fromunicode(e.shortname())
+                yield (hglib.fromunicode(e.rootpath()),
+                       hglib.fromunicode(e.shortname()))
     except:
         f.close()
         raise
